// Copyright 2011 Google Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef NINJA_MAP_H_
#define NINJA_MAP_H_

#include <string.h>

#ifdef _MSC_VER
#include <hash_map>

using stdext::hash_map;

#else

#include <ext/hash_map>

using __gnu_cxx::hash_map;

namespace __gnu_cxx {
template<>
struct hash<std::string> {
  size_t operator()(const std::string& s) const {
    return hash<const char*>()(s.c_str());
  }
};
}

#endif

/// Equality binary predicate for const char*.
struct ExternalStringEq {
  bool operator()(const char* a, const char* b) {
    return strcmp(a, b) == 0;
  }
};

/// Hash functor for const char*.
struct ExternalStringHash {
  size_t operator()(const char* key) const {
#ifdef _MSC_VER
    return stdext::hash<const char*>()(key);
#else
    return __gnu_cxx::hash<const char*>()(key);
#endif
  }
};

/// A template for hash_maps keyed by a const char* that is maintained within
/// the values.  Use like:
///   ExternalStringHash<Foo*>::Type foos;
/// to make foos into a hash mapping const char* => Foo*.
template<typename V>
struct ExternalStringHashMap {
  typedef hash_map<const char*, V, ExternalStringHash, ExternalStringEq> Type;
};

#endif // NINJA_MAP_H_
